Package: DIZutils
Title: Utilities for 'DIZ' R Package Development
Version: 0.1.4
Date: 2026-07-24
Authors@R: c(
    person("Jonathan M.", "Mang", , "jonathan.mang@uk-erlangen.de", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0518-4710")),
    person("Lorenz A.", "Kapsner", role = "aut",
           comment = c(ORCID = "0000-0003-1866-860X")),
    person("Helene Köster", role = "ctb"),
    person("MIRACUM - Medical Informatics in Research and Care in University Medicine", role = "fnd"),
    person("Universitätsklinikum Erlangen, Germany", role = "cph")
  )
Description: Utility functions used for the R package development
    infrastructure inside the data integration centers ('DIZ') to
    standardize and facilitate repetitive tasks such as setting up a
    database connection or issuing notification messages and to avoid
    redundancy.
License: GPL-3
URL: https://github.com/miracum/misc-dizutils
BugReports: https://github.com/miracum/misc-dizutils/issues
Depends: R (>= 3.1.0)
Imports: data.table, DBI (>= 1.1.0), DIZtools, Hmisc, httr, psych,
        RJDBC, RJSONIO, RPostgres, xml2
Suggests: lintr, RPresto, testthat
Encoding: UTF-8
Language: en-US
SystemRequirements: libpq >= 9.0: libpq-dev (deb) or postgresql-devel
        (rpm)
NeedsCompilation: no
Packaged: 2026-07-24 05:33:14 UTC; ubuntu
Author: Jonathan M. Mang [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0518-4710>),
  Lorenz A. Kapsner [aut] (ORCID:
    <https://orcid.org/0000-0003-1866-860X>),
  Helene Köster [ctb],
  MIRACUM - Medical Informatics in Research and Care in University
    Medicine [fnd],
  Universitätsklinikum Erlangen, Germany [cph]
Maintainer: Jonathan M. Mang <jonathan.mang@uk-erlangen.de>
Repository: CRAN
Date/Publication: 2026-07-24 07:10:02 UTC
